function [varargout] = MF_BS_tool_db(reader, param_db, param_est, method_mrq, ...
                                     param_bs, param_test, param_display)
% function [varargout] = MF_BS_tool_db(reader, param_est, method_mrq,
%                                      param_bs, param_test, param_display)
%
% Performs multifractal analysis on all data on a database. Results are
% stored in the BS field of the outputs. mean results are stored in est
% fields of outputs.
%
% Inputs:
%    - reader; structure with parameters
%         -- fun: function that gives access to the database. Each call to
%                 data_fun (no arguments) provides new data. When no more data
%                 is available it returns an empty array.
%         -- ndata: number of elements in database
%   - db_params: structure with parameters
%         -- mean_func: function to compute mean estimate (default = @mean)
%   - Other parameters are forwarded to MF_BS_tool.
%     n_resamp will be disabled.
%
% Outputs:
%   - est
%   - lgt

    %---------------------------------------------------------------------------
    % INIZIALIZATION
    %---------------------------------------------------------------------------

    if ischar (reader)
        % Load from backup
        FLAG_RESTORE_FROM_BACKUP = true;
        % For simplicity, backup will be loaded at the end of the
        % initialization block, overwriting everything.
    else
        % Regular input
        data_fun = reader.fun;
        param_db.ndata = reader.ndata;
        FLAG_RESTORE_FROM_BACKUP = false;
    end

    backup_interval = param_db.backup_interval;
    backup_filename = param_db.backup_filename;
    FLAG_DO_BACKUP = backup_interval > 0;

    % Build a backup command that will be evaluated with eval.
    % This way, the function save will save the current workspace (if this
    % command was a function, it would save its local workspace)
    backup_command = param_db.backup_cmd_fun (backup_filename);

    % Check which multiresolution quantities to use
    do_DWT = ~isempty (find (method_mrq == 1));
    do_LWT = ~isempty (find (method_mrq == 2));
    mrq_fields = [];
    if do_DWT
        mrq_fields = [mrq_fields , {'DWT'}];
    end
    if do_LWT
        mrq_fields = [mrq_fields , {'LWT'}];
    end

    % Check if bootstrap is done
    if isempty (param_bs.ci_method) || (param_bs.n_resamp_1<=1)
        do_BS = 0;
    else
        do_BS=1;
    end

    % Number of estimates:
    [idz, idd, idh, idc] = get_estimate_id (param_est);
    nest = length (idz) + length (idd) + length (idh) + length (idc);

    % Size of database:
    ndata = param_db.ndata;
    %njmax = param_db.njmax;
    njmax = 1;
    % TODO: If these parameters are not provided, I should explore all the
    % database once.

    try
        mean_fun = param_db.mean_fun;
    catch
        mean_fun = @mean;
    end

    % Outputs
    est = [];
    lgt = [];
    stat = [];

    if FLAG_RESTORE_FROM_BACKUP
        % This should notably overwrite data_fun, est,lgt, stat
        load (backup_filename);
    else
        % Read first data to make sure data_fun is not over, and to initialize
        % data_dependent stuff.
        data = data_fun ();

        % Exit if data is empty at this point, since it means data_fun is wrong
        if isempty (data)
            if nargout >= 1
                varargout{1} = est;
            end
            if nargout >= 2
                varargout{2} = lgt;
            end
            if nargout >= 3
                varargout{3} = stat;
            end
            return
        end
        iter = 0;
        initialize_structs ();
    end

    %---------------------------------------------------------------------------
    % DATA PROCESSING
    %---------------------------------------------------------------------------

    % Remember that the first data has already been read in the preamble.
    while ~isempty (data);
        iter = iter + 1;
        [est_tmp, lgt_tmp] = MF_BS_tool (data, param_est, method_mrq,  ...
                                         param_bs, param_test, ...
                                         struct ('fig_num', 0, 'verbosity', 0));
        append_structs ();

        data = data_fun ();  % Get data for next iteration

        if FLAG_DO_BACKUP && mod (iter, backup_interval) == 0
            fprintf ('Saving backup... iter = %i\n', iter)
            eval (backup_command)
        end
    end  % While true

    terminate_structs ();


    %---------------------------------------------------------------------------
    % TERMINATION: plots, tests
    %---------------------------------------------------------------------------
    if param_bs.CI || param_bs.TEST
        for field_cell = mrq_fields
            field = field_cell{1};
            % Use modified param_bs inside of est:
            stat.(field) = MFA_BS_compute_ci_test (est.(field), ...
                                                   param_bs, ...
                                                   param_test);
        end
    end

    if mod (param_display.verbosity, 10) > 0
        make_plots (lgt, est, param_display);
    end

    if mod (floor (param_display.verbosity / 10), 10) > 0
        display_tables (lgt, est, stat);
    end

    %---------------------------------------------------------------------------
    % OUTPUT
    %---------------------------------------------------------------------------
    if nargout >= 1
        varargout{1} = est;
    end
    if nargout >= 2
        varargout{2} = lgt;
    end
    if nargout >= 3
        varargout{3} = stat;
    end

    %---------------------------------------------------------------------------
    % ANCILLARY NESTED FUNCTIONS
    %---------------------------------------------------------------------------
    function initialize_structs ()
    % Iniatialize output est and lgt

        if do_DWT
            est.DWT = [];
            lgt.DWT = [];
            stat.DWT = [];
        end
        if do_LWT
            est.LWT = [];
            lgt.LWT = [];
            stat.LWT = [];
        end

        % Loop over DWT and LWT
        for field_cell = mrq_fields
            field = field_cell{1};

            % Initilize fields that depend on nj with largest possible value,
            % padding with nans.
            lgt.(field).est          = NaN (nest, njmax);
            lgt.(field).supcoef      = NaN (1, njmax);
            lgt.(field).mincoef      = NaN (1, njmax);
            lgt.(field).nj           = NaN (1, njmax);
            lgt.(field).BS.estB      = NaN (ndata, nest, njmax);
            lgt.(field).BS.supcoefB  = NaN (ndata, njmax);
            lgt.(field).BS.mincoefB  = NaN (ndata, njmax);
            lgt.(field).BS.zpB       = NaN (ndata, 1);
            lgt.(field).BS.njB       = NaN (ndata, njmax);
            lgt.(field).BS.p0B.int   = NaN (1, njmax);
            lgt.(field).BS.p0B.noint = NaN (1, njmax);
            if do_BS
                lgt.(field).BS.estBB = NaN (param_bs.n_resamp_1, ...
                                            ndata, nest, njmax);
            end
        end
    end  % function initialize_structs
    %---------------------------------------------------------------------------
    function append_structs ()
    % Add fields of current lgt_tmp and est_tmp to BS structure of output logstat and est
%-        persistent k   % iteration counter
%-        if isempty (k)
%-            k = 0;
%-        end
%-        k = k + 1;

        % Loop over DWT and LWT
        for field_cell = mrq_fields
            field = field_cell{1};

            if size (lgt_tmp.(field).est, 2) > size (lgt_tmp.(field).est, 2)
                expand_logstat (size (lgt_tmp.(field).est, 2));
            end

            jj = 1 : size (lgt_tmp.(field).est, 2);

            lgt.(field).BS.estB(iter, :, jj)  = lgt_tmp.(field).est;
            lgt.(field).BS.supcoefB(iter, jj) = lgt_tmp.(field).supcoef;
            lgt.(field).BS.mincoefB(iter, jj) = lgt_tmp.(field).mincoef;
            lgt.(field).BS.zpB(iter, 1)          = lgt_tmp.(field).zp;
            lgt.(field).BS.njB(iter, jj)      = lgt_tmp.(field).nj;
            lgt.(field).BS.p0B.int(iter)     = lgt_tmp.(field).p0.int;
            lgt.(field).BS.p0B.noint(iter)   = lgt_tmp.(field).p0.noint;

            est.(field).BS.T(iter, :)           = est_tmp.(field).t;
            est.(field).BS.aestB(iter, :)       = est_tmp.(field).aest;
            est.(field).BS.p0B(iter, :)         = est_tmp.(field).p0;
            est.(field).BS.p0nointB(iter, :)    = est_tmp.(field).p0noint;
            est.(field).BS.h_minB(iter, :)      = est_tmp.(field).h_min;
            est.(field).BS.h_min_aestB(iter, :) = est_tmp.(field).h_min_aest;
            est.(field).BS.h_maxB(iter, :)      = est_tmp.(field).h_max;
            est.(field).BS.h_max_aestB(iter, :) = est_tmp.(field).h_max_aest;
            est.(field).BS.zpB(iter, 1)           = est_tmp.(field).zp;


            if do_BS
                lgt.(field).BS.estBB(:, iter, :, jjcur) = lgt_tmp.(field).BS.estB;
                est.(field).BS.TT(:, iter, :) = est_tmp.(field).BS.T;
            end
        end

        % Repeated copy of H_var (it doesn't depend on the realization)
        est.DWT.H_var = est_tmp.DWT.H_var;

        % Clear persistent variable if not needed anymore
%         if k == ndata
%             k = [];
%         end
    end  % function append_structs
    %---------------------------------------------------------------------------
    function expand_logstat (nj)
    % Expands the number-of-scales-dependent matrices in logstat.BS.
    % Precondition: nj is larger than the current size
        for field_cell = mrq_fields
            field = field_cell{1};

            % Sizes of matrices to concatenate:
            sz3 = size (lgt.(field).BS.estB);
            sz3(3) = nj - sz(3);
            sz2 = size (lgt.(field).BS.supcoefB);
            sz2(2) = nj - sz2(2);
            pad2 = NaN (sz2);

            lgt.(field).BS.estB = cat (3, lgt.(field).BS.estB, NaN(sz3));
            lgt.(field).BS.supcoefB = cat (2, lgt.(field).BS.supcoefB, pad2);
            lgt.(field).BS.mincoefB = cat (2, lgt.(field).BS.mincoefB, pad2);
            lgt.(field).BS.njB      = cat (2, lgt.(field).BS.njB     , pad2);
        end
    end
    %---------------------------------------------------------------------------
    function terminate_structs ()
    % Finalize writing of output lgt and est

        param_bs.n_resamp_2 = param_bs.n_resamp_1;
        % Update param_bs.n_resamp_1 inside the loop, since the current value
        % of  "ndata" can be known only for existing fields.

        % Loop over DWT and LWT
        for field_cell = mrq_fields
            field = field_cell{1};

            lgt.(field).est      = squeeze (mean_fun (lgt.(field).BS.estB, 1));
            lgt.(field).supcoef  = mean_fun (lgt.(field).BS.supcoefB, 1);
            lgt.(field).mincoef  = mean_fun (lgt.(field).BS.mincoefB, 1);
            lgt.(field).nj       = mean_fun (lgt.(field).BS.njB, 1);
            lgt.(field).p0.int   = mean_fun (lgt.(field).BS.p0B.int);;
            lgt.(field).p0.noint = mean_fun (lgt.(field).BS.p0B.noint);
            lgt.(field).scale    = 2 .^ (1 : size (lgt.(field).est, 2));
            lgt.(field).zp       = mean_fun (lgt.(field).BS.zpB, 1);
            lgt.(field).BS.Vest  = squeeze (std (lgt.(field).BS.estB, 0, 1));

            est.(field).t          = mean_fun (est.(field).BS.T, 1);
            est.(field).Q          = [];
            est.(field).aest       = mean_fun (est.(field).BS.aestB, 1);
            est.(field).p0         = mean_fun (est.(field).BS.p0B, 1);
            est.(field).p0noint    = mean_fun (est.(field).BS.p0nointB, 1);
            est.(field).h_min      = mean_fun (est.(field).BS.h_minB, 1);
            est.(field).h_min_aest = mean_fun (est.(field).BS.h_min_aestB, 1);
            est.(field).h_max      = mean_fun (est.(field).BS.h_maxB, 1);
            est.(field).h_max_aest = mean_fun (est.(field).BS.h_max_aestB, 1);
            est.(field).zp         = mean_fun (est.(field).BS.zpB, 1);
            est.(field).BS.stdt = squeeze (std (est.(field).BS.T, 0, 1));

            if do_BS
                est.(field).BS.stdT = squeeze (std (est.(field).BS.TT, 0, 1));
            end


            % Write fixed fields
            lgt.(field).Lest = lgt_tmp.(field).Lest;
            lgt.(field).sz_est = lgt_tmp.(field).sz_est;
            lgt.(field).imagedata = lgt_tmp.(field).imagedata;
            lgt.(field).param_est = lgt_tmp.(field).param_est;

            est.(field).LEst = est_tmp.(field).LEst;
            est.(field).sz_est = est_tmp.(field).sz_est;
            est.(field).param_est = est_tmp.(field).param_est;

            param_bs.n_resamp_1 = size (lgt.(field).BS.estB, 1);  % "ndata"

            lgt.(field).param_bs = param_bs;
            est.(field).param_bs = param_bs;
        end
    end  % function terminate_structs
    %---------------------------------------------------------------------------
end % function MF_BS_tool_db
