function [logstat] = MFA_BS_compute_logstat (coef, param)
% function [logstat] = MFA_BS_compute_structure_functions (coef, param_est)
%
% Computes structure functions and cumulants.
%
% Inputs:
%    - coef: structure with fields:
%        * coef.value: {1 x nj} cell array with coefficients for each scale
%          Computations are always performed on coef.value{j}(:)
%    - param_est: struct with estimation parameters.
%
% Output:
%    - logstat: structure with fields:
%        *
%
% February, 2016

% Grand Canonique version
% 2017-08

% RL: For grand canonique, assume all components have the same number of scales.
num_scales = length (coef{1}.value);
fhandle = param.fhandle;

for j = num_scales : -1 : param.j1_ini  % loop scales
    sample = [];

    for ic = 1 : length (coef)
        tmp = abs (coef{ic}.value{j}(:));
        sample{ic} = tmp(isfinite (tmp));
        logstat.supcoef(ic, j) = max (abs (sample{ic}));
        logstat.mincoef(ic, j) = min (abs (sample{ic}));
    end

    % Non-GC versions of flexEstFun don't accept cell array.
    % Convert if this is the case
    if length (coef) == 1
        sample = sample{1};
    end

    [logsf, fdq, fhq, cum] = fhandle (sample, param);

    % - Some of the outputs might be empty depending on param.estimate_select
    logstat.est(:, j) = [logsf(:) ; fdq(:) ; fhq(:) ; cum(:)];
end

% Complete missing scales with NaN:
if param.j1_ini > 1
    idx_new = 1 : param.j1_ini - 1;
    logstat.supcoef(:, idx_new) = NaN;
    logstat.mincoef(:, idx_new) = NaN;
    logstat.est(:, idx_new) = NaN;
end

% Compute coherence on all pairs of data, if 1d :
if ~coef{1}.imagedata
    if length (coef) >= 2
        wcoh = wav_coherence1d (coef, param);
        logstat.coherence = reshape_coherence (wcoh, num_scales)';
    end
else
    if length (coef) >= 2
        wcoh = wav_coherence2d (coef, param);
       logstat.coherence = reshape_coherence (wcoh, num_scales)';
    end
end

% RL Again, assuming this is equal for all components
logstat.nj = coef{1}.nj;
%logstat.Lest = [length(logsf) , length(fdq) , length(fhq) , length(cum)];
logstat.sz_est = {size(logsf),size(fdq) , size(fhq) , size(cum)};
logstat.Lest = cellfun (@prod, logstat.sz_est);
logstat.p0 = p0est (coef{1}, param);
logstat.scale = 2 .^ (1 : num_scales);
logstat.param_est = param;
if param.j2 > num_scales
    logstat.param_est.j2 = num_scales;
end
logstat.zp = coef{1}.zp;

% Recompute correction to store log_Spj in logstat
% RFL: I repeat this computation to keep the code cleaner and avoid adding
% a weird output to DxPx?d
if isfield (coef{1}, 'p') && coef{1}.p > 0  % Exclude DFA
    if coef{1}.imagedata
        [~, ~, log_Spj] = NLcorrPL2d (coef{1}, coef{1}, param);
    else
        [~, ~, log_Spj] = NLcorrPL (coef{1}, coef{1}, param);
    end
    logstat.log_Spj = log_Spj;
end
end  % function

function wcoh = reshape_coherence (wcoh, num_scales)
% Returns a vector with all coherence pairs.
% The input is a matrix of size J x K x K with the between wavelet coefficients
% at each scale for K components.
% Coherences are stored in the lower triangle, with the upper one having NaNs.
%
% Reshape reads first dimension
% (scales) first, so it will produce NaN separated groups of num_scales
% coefficients.

    wcoh = reshape (wcoh, 1, []);
    wcoh = wcoh(~isnan (wcoh));
    wcoh = reshape (wcoh, num_scales, []);
end
