function [Elogmuqj, f_Dq, f_hq, Cp] = flexEstFun_MFA_GC(coefs, param)
% function [Elogmuqj, f_Dq, f_hq, Cp]] = flexEstFun_MFA(Absdqk, param)
%
% Calculates Structure Functions (SF), spectrum and log cumulants for grand-canonical multifractal formalism
%
% -- INPUT
%   coefsk  -   cell array with |d_X(j,.)| or L_X(j,.) for each component,
%   param   -   structure with parameters
%       param.q1, param.q2 : vectors with moments q to calculate
%       param.cum    : highest order of Cumulant to calculate
%       param.estimate_select : what is to be calculated
%                       number xyz
%                       x : SF zeta(q) [0 or 1]
%                       y : SF D(h) [0 or 1]
%                       y : SF Cp [0 or 1]
%                       e.g. param.estimate_select = 001 calculates only Cp,
%                            param.estimate_select = 110 calculates SF zeta(q) and D(h)
%
%  -- OUTPUT:
%   Elogmuqj : matrix nq1 x nq2
%   f_Dq     : matrix nq1 x nq2
%   f_hq     : matrix nq1 x nq2 x 2
%   Cp       : vector 1 < (2 * cum) + 3
%
%  Output that are not requested by param.estimate_select are returned empty
%
% Herwig Wendt, Lyon, 2006 - 2008
% Stéphane Roux, Lyon, 2007
% Roberto Leonarduziz, Hong Kong, 2017


% RL 2017-08
% Enable only two components for the moment, for easy first adaptation of code
% and checking that everything works
if length (coefs) ~= 2
    error ('Only 2 components supported for the moment')
end

Absdqk = coefs{1}(:)';
Absdqk2 = coefs{2}(:)';

SORTOUT = 1; % sort out small values
thresh = eps; %1e-10;
if SORTOUT
    Absdqk = Absdqk(Absdqk >= thresh);
    Absdqk2 = Absdqk2(Absdqk2 >= thresh);
    if length (Absdqk) ~= length (Absdqk2)
        lenmin = min (length (Absdqk), length (Absdqk2));
        Absdqk = Absdqk(1 : lenmin);
        Absdqk2 = Absdqk2(1 : lenmin);
    end
end

% Check what needs to be computed
[ZQ, DH, CP] = which_estimates (param.estimate_select);


%% how many partition functions

q1 = param.q;
nq1 = length(q1);
if isfield (param, 'q2') && ~isempty (param.q2)
    q2 = param.q2;
    nq2 = length (q2);
else
    q2 = q1;
    nq2 = nq1;
end

Nq = 0;
if ZQ; Nq = Nq + nq1 * nq2    ; end;
if DH; Nq = Nq + 3 * nq1 * nq2; end;
if CP; Nq = Nq + param.cum * 3; end;

% Default output
Elogmuqj = [];
f_Dq     = [];
f_hq     = [];
Cp       = [];


if ZQ || DH  % if not only Cumulants

  %% structure functions and Moments. f_Dq and f_hq WITHOUT LOOP
  %% This version is faster than the loop for length(q)>3 and length(Absdqk)>10 (~0.7 times the loop)
  ncoef = length (Absdqk);

  S1 = repmat (Absdqk, nq1, 1);
  Q1 = repmat (q1(:), 1, ncoef);

  ncoef2 = length (Absdqk2);
  S2 = repmat (Absdqk2, nq1, 1);

  for iq=1:length(q2)
    detail_kq = S1.^Q1 .* S2.^q2(iq);
    Elogmuqj(:,iq) = log2 (mean (detail_kq, 2))';

    % D(h): Fun = 2, 3, 6, 7
    if DH
      sum_detail_kq = sum (detail_kq, 2);

      f_Dq(:,iq) = ...
          sum(detail_kq .* log2 (detail_kq ./ repmat (sum_detail_kq ,1, ncoef)), ...
              2) ./ ...
          sum_detail_kq + ...
          log2 (ncoef);
      f_hq(:, iq, 1) = sum(detail_kq .* log2(S1),2) ./ sum_detail_kq;
      f_hq(:, iq, 2) = sum(detail_kq .* log2(S2),2) ./ sum_detail_kq;
    end
  end
end

if CP   % compute  cumulants
    %- calcul de C_{i,0} et C_{0,i}
    Cp1 = compute_cum_1d (log (Absdqk), param.cum);
    Cp2 = compute_cum_1d (log (Absdqk2), param.cum);

    if param.cum > 1
        %- calcul de C_{1,1}
        detail_kq = log (Absdqk).*log (Absdqk2);
        Cp3(:,1) = mean (detail_kq)  -Cp1(:,1) .* Cp2(:,1);
    end

    if param.cum > 2
        %- calcul de C_{2,1}
        detail_kq = log (Absdqk).^2 .* log (Absdqk2);
        Cp3(:,2) = mean (detail_kq) - Cp1(:,2) .* Cp2(:,1) - ...
            2 * Cp3(:,1) .* Cp1(:,1) - Cp1(:,1).^2 .* Cp2(:,1);

        %- calcul de C_{1,2}
        detail_kq = log (Absdqk) .* log (Absdqk2).^2;
        Cp3(:,3) = mean (detail_kq) - Cp2(:,2) .* Cp1(:,1) - ...
            2 * Cp3(:,1) .* Cp2(:,1) - Cp2(:,1).^2 .* Cp1(:,1);
    end

    Cp = reorder_cumulants (param.cum, Cp1, Cp2, Cp3);
end

end % function flexEstFun_MFA_GC.m
%----------------------------------------------------------------------
function [Cp] = compute_cum_1d (detail, ncum)
    Cp(:,1) = mean (detail) ;
    if ncum > 1
        Cp(:,2) = mean (detail.^2) - Cp(:,1).^2 ;
    end
    if ncum > 2
        Cp(:,3) = mean (detail.^3) - 3 * Cp(:,2) .*Cp (:,1) - Cp(:,1).^3 ;
    end
    if ncum > 3
        Cp(:,4) = mean (detail.^4) - 4 * Cp(:,3) .* Cp(:,1) - ...
                  3 * Cp(:,2).^2 - 6 * Cp(:,2) .* Cp(:,1).^2 - Cp(:,1)^4;
    end
    if ncum > 4
        Cp(:,5) = mean(detail.^5) - 5 * Cp(:,4) .* Cp(:,1) - ...
                  10 * Cp(:,3) .* Cp(:,2) - 10 * Cp(:,3) .* Cp(:,1).^2 - ...
                  15 * Cp(:,2).^2 .* Cp(:,1) - 10 * Cp(:,2) .* Cp(:,1).^3 - ...
                  Cp(:,1).^5 ;
    end
end
%----------------------------------------------------------------------
function [new] = reorder_cumulants (ncum, c1, c2, ccross)
% Create an array of cumulants, ordered as:
%    c10, c01, c20, c11, c02, c30, c21, c12, c03, ...
%
% Inputs:
%    - ncum: order of cumulants computed
%    - c1, c2: univariate cumulants of first and second component
%    - ccross: cross cumulants, already ordered

    inew = 1;
    i1 = 1;
    i2 = 1;
    icross = 1;

    for m = 1 : ncum
        % 1) Write current cumulant of first component
        new(inew) = c1(i1);
        i1 = i1 + 1; inew = inew + 1;

        % 2) Write the (m-1) cross-cumulants of order m
        for n = 1 : m - 1
            new(inew) = ccross(icross);
            icross = icross + 1;
            inew = inew + 1;
        end

        % 3) Write current cumulant of second component
        new(inew) = c2(i2);
        i2 = i2 + 1;
        inew = inew + 1;
    end
end