% Simple demo on the use of MF_BS_tool.
% Does wavelet leader based multifractal analysis, using boostrap to compute
% confidence intervals on estimates and perform hypothesis tests
%
% Roberto Leonarduzzi
% 2016-07-04

% Add root folder of toolbox to path (subfolders are added automatically)
addpath ..

% Add examplesest to path
addpath ./WLBMF_example_data

%% Select example data

MFprocess = 2;

switch MFprocess
    case 1; % fbm, N=4096
        load fbm08n4096;
        N = length (data);
        figure (11111); clf
        plot (data)
        TSTR = 'A realization of fractional Brownian motion (H=0.8, N=4096)';
    case 2; % mrw, N=32768
        load mrw07005n32768;
        N = length (data);
        figure (11111); clf
        plot (data)
        TSTR = 'A realization of multifractal random walk (c_1=0.75, c_2=-0.05, N=32768)';
    case 3; % cpc ln, N=1024x1024
        load cmcLN2d_00125_0025_n1024;
        N = length (data);         
        figure (11111); clf
        surf (data)
        TSTR = 'A realization of 2d compound poisson cascade with log-normal multipliers (c_1=0.0125, c_2=-0.025, N=1024x1024)';
        data = data (1 : 256, 1 : 256);
end
title(TSTR); 
grid on;



%% Setup analysis

% Create an object of class MF_BS_tool_inter to perform analysis
mf_obj = MF_BS_tool_inter;

% All analysis parameters are properties of this class that can be accessed
% with operator '.'

%===============================================================================
% General parameters 
%===============================================================================

% Select order of Daubechies wavelet:
mf_obj.nwt = 3;

% Select which multiresolution quantities are used:
mf_obj.method_mrq = [1 2];    % 1: Wavelet coefficients, 2: wavelet leaders

% Set up scaling range:
mf_obj.j1      = 3;
mf_obj.j2      = 12;

% Use a convenient set of orders q
mf_obj.q       = build_q_log (0.01, 5, 10);

% Compute the first three log-cumulants
mf_obj.cum     = 3;

% Indicate pseudo-fractional-integration order
mf_obj.gamint = 0;

% Indicate figure number to be used by the tool
mf_obj.fig_num = 100;

% Set up versbosity level: show figures and tables:
mf_obj.verbosity = 11;   % Number xy, with x,y either 0 or 1
                         %    x = 1: display figures
                         %    y = 1: display tables with estimates   
%===============================================================================
% Bootstrap parameters 
%===============================================================================

% Select bootstrap type:
mf_obj.bs_type = 1;  % 0: time block boostrap
                     % 1: time-scale block bootstrap

% Set block size depending on bootstrap type
if mf_obj.bs_type == 1
    mf_obj.block_size = floor (N / 32);
else
    mf_obj.block_size = mf_obj.nwt * 2;
end

% Indicate number of primary and secondary resamples:
mf_obj.n_resamp_1 = 100;
mf_obj.n_resamp_2 = 0;

% Activate computation of confidence intervals:
mf_obj.CI = 1;

% Select methods to compute confidence intervals:
%[1-Normal; 2-Basic; 3-Percentile; 4-Studentised; 5-Adjusted Basic; 6-Adjusted Percentile]
mf_obj.ci_method = [2 3];  

%===============================================================================
% Tests parameters 
%===============================================================================

% Activate computation of tests on log-cumulants, based on confidence intervals:
mf_obj.TEST = 1;

% Select type of null hypothesis:
% T=T0 Types [  HA: T>T0;   HA: T<T0;   H0: |T-T0|=0;   H0: T-T0=0  ]
mf_obj.null_type = 4;

% Select vaues T0 of null hypothesis
mf_obj.null_param = [0.7 0 0];

% Select significance level:
mf_obj.alpha = 0.05;

%% Perform analysis

% The input can be a 1d or 2d array.
mf_obj.analyze (data);

%% Access results

% Get estimates for wavelet coefficients and leaders:
t_dwt = mf_obj.est.DWT.t;  % (size nest x 1)
t_lwt = mf_obj.est.LWT.t;  % (size nest x 1)

% Get logscale diagrams for wavelet coefficients and leaders:
lsd_dwt = mf_obj.logstat.DWT.est;  % (size nest x nj)
lsd_lwt = mf_obj.logstat.LWT.est;  % (size nest x nj)

% The nest estimates are ordered as [zeta(q), D(q), h(q), c_m]
% Each one can be accessed individually as:

zid = mf_obj.get_zid ();  % Indices of zeta(q)
hid = mf_obj.get_hid ();  % Indices of h(q)
Did = mf_obj.get_Did ();  % Indices of D(q)
cid = mf_obj.get_cid ();  % Indices of c_p

zq = mf_obj.est.LWT.t(zid);  % Estimates of zeta(q)
hq = mf_obj.est.LWT.t(hid);  % Estimates of h(q)
Dq = mf_obj.est.LWT.t(Did);  % Estimates of D(q)
cp = mf_obj.est.LWT.t(cid);  % Estimates of c_p

% Structure functions:
sf_zq  = mf_obj.logstat.LWT.est(zid, :);  % Structure functions log_2 S(q,j)
sf_hq  = mf_obj.logstat.LWT.est(hid, :);  % "Structure functions" log_2 U(q,j)
sf_Dq  = mf_obj.logstat.LWT.est(Did, :);  % "Structure functions" log_2 V(q,j)
sf_cum = mf_obj.logstat.LWT.est(cid, :);  % Cumulants C_m(j)

