% Simple demo on the use of MF_BS_tool.
% Does wavelet leader based multifractal analysis, using an automated
% bootstrap based detection of the scaling range
%
% Roberto Leonarduzzi
% 2016-07-04

% Add root folder of toolbox to path (subfolders are added automatically)
addpath ..

% Add examplesest to path
addpath ./WLBMF_example_data

%% Select example data

MFprocess = 2;

switch MFprocess
    case 1; % fbm, N=4096
        load fbm08n4096;
        N = length (data);
        figure (11111); clf
        plot (data)
        TSTR = 'A realization of fractional Brownian motion (H=0.8, N=4096)';
    case 2; % mrw, N=32768
        load mrw07005n32768;
        N = length (data);
        figure (11111); clf
        plot (data)
        TSTR = 'A realization of multifractal random walk (c_1=0.75, c_2=-0.05, N=32768)';
    case 3; % cpc ln, N=1024x1024
        load cmcLN2d_00125_0025_n1024;
        N = length (data);         
        figure (11111); clf
        surf (data)
        TSTR = 'A realization of 2d compound poisson cascade with log-normal multipliers (c_1=0.0125, c_2=-0.025, N=1024x1024)';
        data = data (1 : 256, 1 : 256);
end
title(TSTR); 
grid on;



%% Setup analysis

% Create an object of class MF_BS_tool_inter to perform analysis
mf_obj = MF_BS_tool_inter;

%===============================================================================
% General parameters 
%===============================================================================

% All analysis parameters are properties of this class that can be accessed
% with operator '.'

% Select order of Daubechies wavelet:
mf_obj.nwt = 3;

% Select which multiresolution quantities are used:
mf_obj.method_mrq = [1 2];    % 1: Wavelet coefficients, 2: wavelet leaders


% Use a convenient set of orders q
mf_obj.q       = build_q_log (0.01, 5, 10);

% Compute the first three log-cumulants
mf_obj.cum     = 3;

% Indicate pseudo-fractional-integration order
mf_obj.gamint = 0;

% Indicate figure number to be used by the tool
mf_obj.fig_num = 100;

% Set up versbosity level: show figures:
mf_obj.verbosity = 1;   % Number xy, with x,y either 0 or 1
                        %    x = 1: display figures
                        %    y = 1: display tables with estimates   

%===============================================================================
% Scaling range selection setup
%===============================================================================

% Set up search region for scaling range (inf indicates larges scale available):
mf_obj.j1      = [1 inf];
mf_obj.j2      = [1 inf];

% Bootstrap needs to be enabled:
mf_obj.bs_type = 1;  % Use time-scale block bootstrap
mf_obj.n_resamp_1 = 100;  % Use 100 resamples

%% Perform analysis

% The input can be a 1d or 2d array.
mf_obj.analyze (data);

%% Access results

%===============================================================================
% Detected scaling range
%===============================================================================

% The detected scaling range is stored in the param_est field of the est and
% logstat outputs:

j1_det = mf_obj.est.LWT.param_est.j1;
j2_det = mf_obj.est.LWT.param_est.j2;

%===============================================================================
% Estimates
%===============================================================================


% Get estimates for wavelet coefficients and leaders:
t_dwt = mf_obj.est.DWT.t;  % (size nest x 1)
t_lwt = mf_obj.est.LWT.t;  % (size nest x 1)

% Get logscale diagrams for wavelet coefficients and leaders:
lsd_dwt = mf_obj.logstat.DWT.est;  % (size nest x nj)
lsd_lwt = mf_obj.logstat.LWT.est;  % (size nest x nj)

% The nest estimates are ordered as [zeta(q), D(q), h(q), c_m]
% Each one can be accessed individually as:

zid = mf_obj.get_zid ();  % Indices of zeta(q)
hid = mf_obj.get_hid ();  % Indices of h(q)
Did = mf_obj.get_Did ();  % Indices of D(q)
cid = mf_obj.get_cid ();  % Indices of c_p

zq = mf_obj.est.LWT.t(zid);  % Estimates of zeta(q)
hq = mf_obj.est.LWT.t(hid);  % Estimates of h(q)
Dq = mf_obj.est.LWT.t(Did);  % Estimates of D(q)
cp = mf_obj.est.LWT.t(cid);  % Estimates of c_p

% Structure functions:
sf_zq  = mf_obj.logstat.LWT.est(zid, :);  % Structure functions log_2 S(q,j)
sf_hq  = mf_obj.logstat.LWT.est(hid, :);  % "Structure functions" log_2 U(q,j)
sf_Dq  = mf_obj.logstat.LWT.est(Did, :);  % "Structure functions" log_2 V(q,j)
sf_cum = mf_obj.logstat.LWT.est(cid, :);  % Cumulants C_m(j)

