%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     DEMO OF BOOTSTRAP MFA ANALYSIS with MF_BS_tool.m:
%                demo_MF_BS_tool_singul.m
%     Wavelet coefficient and leader based estimation of multifractal attributes
%     Wavelet domain block bootstrap based confidence limits and significance tests
% 
% Herwig Wendt, Lyon, 2006 - 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Please refer to the references on my web site (and the below given
% references) for details on the method.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If you use the toolbox, please quote:
%
%     @ARTICLE{WendtSPM2007,
%       author = {Herwig Wendt and Patrice Abry and St??phane Jaffard},
%       title = {Bootstrap for Empirical Multifractal Analysis},
%       journal = {IEEE Signal Proc. Mag.},
%       volume = {24},
%       number = {4},
%       pages = {38--48},
%       year = {2007},
%     }
%
% and (or)
%
%     @ARTICLE{WendtSP2009,
%       author = {Herwig Wendt and St?phane G. Roux and Patrice Abry and St??phane Jaffard},
%       title = {Wavelet leaders and bootstrap for multifractal analysis of images},
%       journal = {Signal Proces.},
%       volume = {89},
%       pages = {1100--1114},
%       year = {2009},
%     }
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear variables
pathMF_BS_tool_singul;         % add subpaths of toolbox
 addpath ./WLBMF_example_data/

%% CHOOSE EXAMPLE
MFprocess = 2;
% case 1: fbm,       N=4096         (1d, self-similar/monofractal)
% case 2: mrw,       N=32768        (1d, multifractal)
% case 3: cpc ln, N=1024 x 1024     (2d, multifractal)

switch MFprocess
    case 1; % fbm, N=4096
      load fbm08n4096; 
      N = length (data);
      TSTR = 'A realization of fractional Brownian motion (H=0.8, N=4096)';
      plot_fun = @plot;
  case 2; % mrw, N=32768
    load mrw07005n32768;
    N = length (data);
    TSTR = 'A realization of multifractal random walk (c_1=0.75, c_2=-0.05, N=32768)';
    plot_fun = @plot;
  case 3; % cpc ln, N=1024x1024
    load cmcLN2d_00125_0025_n1024;
    alpha=0.015;
    data = fi_2d_fft_pc (data, alpha); 
    data = real(data) ; 
    data=data(1:256,1:256);
    TSTR = 'A realization of 2d compound poisson cascade with log-normal multipliers (c_1=0.0125, c_2=-0.025, N=1024x1024)';
    plot_fun = @surf;
end
%         ff=fspecial('gaussian',7,0.5); ff=ff/sqrt(sum(ff(:).^2));data=filter2(ff,data); 
%         ff=fspecial('disk',1); ff=ff/sqrt(sum(ff(:).^2)); data=filter2(ff,data); 
        
figure(11111); clf; 
plot_fun (data); 
title(TSTR);
grid on;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  PARAMETER SETUP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--- Analysis Methods to use
method_mrq = [1 2];     %- 1 - DWT  (discrete wavelet transform coefficients)
                        %- 2 - PWT  (p-Leaders)

%===============================================================================
% Estimation parameters
%===============================================================================
param_est.nwt    = 3;              %- vanishing moments wavelet (Daubechies')
param_est.gamint = 0;              %- Wavelet domain fractional integration by gamint
param_est.j1     = 3;              %- j1,j2 - scaling range
param_est.j2     = 15;
param_est.wtype  = 0;              %- linear regression type: 
                                   %-      0 - ordinary
                                   %-      1 - weighted nj(j)
                                   %-      2 - weighted with estimated variance
param_est.estimate_select = 111;   %-       number xyz - what is to be calculated:  
                                   %-       x : zeta(q) [0 or 1]    (scaling exponents)
                                   %-       y : D(h) [0 or 1]       (multifractal spectrum)
                                   %-       z : cp [0 or 1]         (log-cumulants)
                                   %-       e.g. 001 calculates only Cp, 
                                   %-            110 calculates SF zeta(q) and D(h)
param_est.cum    = 3;              %- Highest order of log cumulants cp to estimate
param_est.q      = -5 : 1: 5;      %- Moments to estimate (zeta(q) and D(h))
param_est.p      = inf;            %- norm of p-leaders (p = inf => wavelet leaders)

param_est.plead_corr = 2;          %- Select use of scaling correction term:

param_est.j1_ini     =  1;         %- scale where leaders start to be computed
param_est.sym        = 0;          %- symmetrize wavelet (1d only)
                                   %- 0: don't use correction
                                   %- 1: use correction if $p \neq \infty$
                                   %- 2: use correction if $p \neq \infty$ and $\eta(p) \geq 0$

% Parameters for different exponents:
param_est.delta_gam = +0.05;     % differential fractional integration (+) or derivation (-)
param_est.delta_p = +0.02;       % delta 1/ for derivative d/d(1/p) 
param_est.type_exponent = 0;     %- 0: p-exponent.            
                                 %- 1: oscillation exponent.      [delta_gam used]
                                 %- 2: lacunary exponent.         [delta_p used]
                                 %- 3: cancellation exponent.     [delta_gam used]
                                 %-       NOTE: need gamint=0

param_est.fhandle = @flexEstFun_MFA;

%===============================================================================
% Bootstrap parameters 
%===============================================================================
param_bs.bs_type = 0;        % time - scale block [1] or time block [0] bootstrap
param_bs.n_resamp_1 = 10;     % # primary resamples - NO BOOTSTRAP (ESTIMATION ONLY): B1=0;
param_bs.n_resamp_2=10;       % # double bootstrap resamples
param_bs.CI=1;               % calculate confidence intervals
param_bs.TEST=1;             % calculate tests
if param_bs.bs_type          % Block Length
    param_bs.block_size=floor(N/32);     
else
    param_bs.block_size=2 * param_est.nwt; 
    if param_est.nwt >= 1000;
        param_bs.block_size = 200;
    end
end

param_bs.ci_method=[1 2 3 4];        % Methods to be used [1-Normal; 2-Basic; 3-Percentile; 4-Studentised; 5-Adjusted Basic; 6-Adjusted Percentile]
param_bs.alpha=0.1;                % Significance Level (1-alpha)
param_bs.flag_bs_range=1;          % 1: do not do Bootstrap on scales that are not implied in regression (use 0 only if you need confidence intervals for scales not involved in regressions)

%===============================================================================
% Test parameters 
%===============================================================================
param_test.null_param = zeros(1,param_est.cum); % Null Hypothesis Parameter (for cp only)
param_test.null_type = [4];                     % Null Hypothesis H0: T=null_param Types [  HA: T>null_param;   HA: T<null_param;   H0: |T-null_param|=0;   H0: T-null_param=0  ]

%--- VERBOSITY PARAMETERS
param_display.fig_num = 100;           % 0 - no Figures
param_display.verbosity=21;          % Verbosity level:
                    %   0 : batch mode
                    %   1 : estimate, CI, Test table
                    %       Figure estimate if fig_num
                    %   11: Figure estimate if fig_num
                    %   2 : 1 + Figure LogScale if fig_num
                    %   21: 11+ Figure LogScale if fig_num
                    %   3 : 2 + interactive mode (change Estimates, j1, j2, wtype,
                    %           Analysis method, alpha, Text output, Figures)
param_display.bs_plot_type = 'ci';
                    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% INITIALISATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rand('state',sum(100*clock));
% % check and write parameters:
% checkParam_MF_BS_tool_singul;
% [param_est, param_bs, param_test]=MF_BS_tool_param_singul(nwt,gamint,delta_gam,delta_p,type_exponent,j1,j2,wtype,estimate_select,cum,q,n_resamp_1,n_resamp_2,block_size,ci_method,alpha,flag_bs_range,null_type,null_param,bs_type,CI,TEST,p,j1_ini,sym,plead_corr);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ANALYSIS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic;
[est,logstat, stat, mrq] = MF_BS_tool(data, param_est, method_mrq, param_bs, param_test, verbosity, fig_num);
toc;


% %% Reuse logstat
% tic;
% MF_BS_tool(logstat, param_est, method_mrq, param_bs, param_test, verbosity, fig_numum + 100);
% toc;
% 
% %% Reuse mrw
% tic;
% MF_BS_tool(mrq, param_est, method_mrq, param_bs, param_test, verbosity, fig_num + 200);
% toc;




return

% OUTPUT:
%
% estimates are organized as [zeta(q), D(q), h(q), cp]
%-- est: est.DWT, est.LWT
%%  est.DWT.t(estid)            - estimates
%   est.DWT.stdt(estid)         - bootstrap standard deviation
%   est.DWT.T(bsid,estid)       - bootstrap estimates
%   est.DWT.aest(estid)         - regression instersect estimation
%   est.DWT.Q(estid)            - regression quality estimation
%   est.DWT.LEst                - length of estimates zeta(q), D(q), h(q), cp
%
%-- conf: conf.DWT, conf.LWT
%   conf.DWT{estid}{Methodid}.name  - Method name
%%  conf.DWT{estid}{Methodid}.lo    - lower confidence limit
%%  conf.DWT{estid}{Methodid}.hi    - upper confidence limit
%
%-- signif: signif.DWT, signif.LWT
%   signif.DWT{estid}{Methodid}{Typeid}.name      - Method name
%   signif.DWT{estid}{Methodid}{Typeid}.reject    - Test decision
%%  signif.DWT{estid}{Methodid}{Typeid}.plevel    - plevel
%
%-- logstat
%%  logstat.DWT.est(estid, j)           - Structure Functions
%   logstat.DWT.nj(j)                   - number of coefficients at each scale
%   logstat.DWT.estB(bsid, estid, j)    - Bootstrap structure functions
%   logstat.DWT.Vest(estid, j)          - Bootstrap standard deviation
%   logstat.DWT.j1                      - scaling range [j1,j2] used
%   logstat.DWT.j2
%   logstat.DWT.scale                   - scale 2^j
%   logstat.DWT.imagedata               - imaghe or time series
%   logstat.parBSestDyad            - bootstrap parameters used
%   logstat.j1min                   - minimal scale for regression (bootstrap)
%   logstat.j2max                   - maximal scale for regression
%   logstat.cum                     - number of cumulants calculated
%   logstat.Norm                    - Wavelet norm used
%   logstat.nwt                  - vanishing moments wavelet used
%   logstat.EstFun                  - Estimation fuction parameter
%   logstat.q                       - moments q
%   logstat.flag_bs_range                   - flag_bs_range: Bootstrap on scale not used in regression
%   logstat.bs_type                     - time scale block bootstrap
%
%% Analyzed function in Lp for p0 but not in Lp for p0+epsilon (epsilon>0)
% -- est.DWT.p0                     - function in Lp with p=p0 after fractional integration
% -- est.DWT.p0noint                - function without fractional integration in Lp with p=p0noint
% --> NOTE: p0=-1 if the function is not in any Lp
%% Minimum and maximum regularity estimation
% -- est.DWT.
% h_min                             - minimum Hoelder exponent after fractional integration
% h_minnoint                        - minimum Hoelder exponent without fractional integration (original function)
% h_minL                            - minimum Hoelder exponent after fractional integration: Leaders (EXPERIMENTAL) 
% h_max                             - largest Hoelder exponent after fractional integration (Leaders based estimation)
% h_maest                           - corresponding linear regression intersects

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ANALYSIS bis 
% You can reuse structure functions stored in "LogScale" to test different
% hypothesis interactively
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
