function make_plots_gc (logstat, est, param_display)

% Makes the plots of all results for Grand Canonical formalism
%
% First version 2017-08:
% - Plot only the LWT field because the number of plots is huge.
% - For now Grand Canonical doesn't have bootstrap enable, so don't worry
% too much about that.

%===============================================================================
%--- SETUP
%===============================================================================

% --- Constants ---
loge = log2 (exp (1));

% --- Determine which methods were used ----
flag_DWT = isfield (logstat, 'DWT');
flag_LWT = isfield (logstat, 'LWT');

if ~flag_DWT | ~flag_LWT
    error ('Either DWT or LWT must be computed')
end

if isfield (logstat.DWT, 'param_est')
    param_est = logstat.DWT.param_est;
    imagedata = logstat.DWT.imagedata;
else
    param_est = logstat.LWT.param_est;
    imagedata = logstat.LWT.imagedata;
end

if isfield (logstat.DWT, 'BS')
    param_bs = logstat.DWT.param_bs;
elseif isfield (logstat.LWT, 'BS')
    param_bs = logstat.LWT.param_bs;
else
    param_bs = [];
end

if ~isempty (param_bs)
    doBS = param_bs.n_resamp_1 > 0;
else
    doBS = false;
end

% --- Determine what estimates where computed ---
[ZQ, DH, CP] = which_estimates (param_est.estimate_select);
[idx_zq, idx_dq, idx_hq idx_cum] = get_estimate_id (param_est);

% Selected method for estimates: use LWT if present, fall back to DWT otherwise
if isfield (est, 'LWT')
    sel_field = 'LWT';
    im_sel = 2;
else
    sel_field = 'DWT';
    im_sel = 1;
end
lgt_mat = reshape_estimates_GC (logstat.(sel_field));
est_mat = reshape_estimates_GC (est.(sel_field));

% reshape DWT logstat also, to show moments.
lgt_mat_dwt = reshape_estimates_GC (logstat.DWT);
est_mat_dwt = reshape_estimates_GC (est.DWT);

ncomp = 2;
ncum = length (est_mat.t.cp);
[nq1, nq2] = size (est_mat.t.zq);

% --- Setup figure flags ---
if (ZQ|CP)
    PlotLogScale=1;
else
    PlotLogScale=0;
end

PlotResult=1;
figs=PlotLogScale+PlotResult;


% --- Method dependent strings for titles ---
meth_name = [];
meth_str = [];
if flag_DWT
    meth_name = [meth_name {'DWT'}];
    meth_str = [meth_str {'DWT'}];
end
if flag_LWT
    meth_name = [meth_name {'LWT'}];
    meth_str = [meth_str {sprintf('PWT p=%g', param_est.p)}];
end
nmeth = length (meth_name);

switch param_est.type_exponent,
  case 0
    expstr=['hp'];
    parstr=[''];
  case 1
    expstr=['Oscillation'];
    parstr=['d\gamma=',num2str(param_est.delta_gam)];
  case 2
    expstr=['Lacunary'];
    parstr=['d 1/p=',num2str(param_est.delta_p)];
  case 3
    expstr=['Cancellation'];
    parstr=['d\gamma=',num2str(param_est.delta_gam)];
end

% Initialize plot_data with common parameters
switch param_display.bs_plot_type
  case 'ci'
    plot_data.bs_plot_logscale_fun = @plot_logscale_ci;
    plot_data.bs_plot_xy_fun       = @plot_xy_ci;
end

% id strings for cumulants:
cum_str = [];
idx_curr = 1;
for m = 1 : param_est.cum
    one = m;
    two = 0;
    for k = 1 : m + 1
        cum_str{idx_curr} = sprintf ('c_{%i,%i}', one, two);
        idx_curr = idx_curr + 1;
        one = one - 1;
        two = two + 1;
    end
end

%-for ic = 1 : param_est.cum
%-    cum_str{end + 1} = sprintf ('c_{%i,0}', ic);
%-end
%-for ic = 1 : param_est.cum
%-    cum_str{end + 1} = sprintf ('c_{0,%i}', ic);
%-end
%-cum_str{end + 1} = 'c_{1,1}';
%-cum_str{end + 1} = 'c_{2,1}';
%-cum_str{end + 1} = 'c_{1,2}';

%-------------------------------------------------------------------------------
% BOXPLOTS OF CUMULANTS
% GC: boxplots of all cumulants and cross-cumulatns for selected component
%-------------------------------------------------------------------------------
if PlotResult && CP
    figure (param_display.fig_num); clf
    set (gcf, 'Name', 'Estimates: log-cumulants')
    nrow = param_est.cum;
    ncol = param_est.cum + 1;  % Number of cumulants for largest order
    idx_curr = 0;  % Index of current cumulant

    for m = 1 : param_est.cum   % rows, cumulant order
        for k = 1 : m + 1  % cols, cumulants of order m
            idx_curr = idx_curr + 1;
            subplot (nrow, ncol, k + (m - 1) * ncol)

            % Get index of current cumulant
            plot_data.yy = est_mat.t.cp(idx_curr);
            if doBS
                plot_data.yy_bs = est_mat.BS.t.cp(:, idx_curr);
            else
                plot_data.yy_bs = [];
            end

            plot_boxplot (gca, plot_data, param_est, param_bs);
            ylabel (cum_str{idx_curr})
            title (sprintf ('%s %s', meth_str{im_sel}, expstr));
        end
    end
end
%-------------------------------------------------------------------------------
% SCALING FUNCTION AND MULTIFRACTAL SPECTRUM
%-------------------------------------------------------------------------------
if PlotResult && (DH | ZQ)
    hf = figure (param_display.fig_num + 1); clf
    set (hf, 'Name', 'Estimates: scaling function')
    %-----------------------------------
    %- zeta(q)
    if ZQ
        subplot (1, 2, 1)
        [qq2, qq1] = meshgrid (param_est.q2, param_est.q);
        plot_data.xx = qq2;
        plot_data.yy = qq1;
        plot_data.zz = est_mat.t.zq;

        % Set stuff to create labels:
        plot_data.x_str= 'q_1';
        plot_data.y_str = 'q_2';
        plot_data.z_str = '\zeta';

        %plot_data.p0noint = est.(meth_name{im}).p0noint;
        %plot_data.p0 = est.(meth_name{im}).p0;

        plot_surf (plot_data, param_est, param_bs);
        %xlabel ('q');
        %ylabel ('\zeta(q)');
        %title (sprintf ('%s, \\zeta(q), %s', meth_str{im}, expstr))
    end
    %-----------------------------------
    %- D(h)
    if DH
        subplot (1, 2, 2)
        plot_data.xx = est_mat.t.hq(:, :, 1);
        plot_data.yy = est_mat.t.hq(:, :, 2);
        plot_data.zz = est_mat.t.Dq;

        % Set stuff to create labels:
        plot_data.x_str= 'h_1';
        plot_data.y_str = 'h_2';
        plot_data.z_str = 'D';
        % Set target limits for good perspective of spectrum.
        % They will be enlarged if needed.
        plot_data.z_lim = [0 , 1 + param_est.imagedata];
        plot_data.x_lim = [0 , 1];
        plot_data.y_lim = [0 , 1];

        % Plot projections for the spectrum
        plot_data.plot_projections = true;

        %plot_data.p0noint = est.(meth_name{im}).p0noint;
        %plot_data.p0 = est.(meth_name{im}).p0;

        plot_surf (plot_data, param_est, param_bs);

%         xl = xlim;
%         xl(1) = min (xl(1), 0);
%         xl(2) = max (xl(2), 1.5);
%         xlim (xl)
%         yl = ylim;
%         yl(1) = 0;
%         yl(2) = max (yl(2), (imagedata + 1) * 1.05);
%         ylim (yl)
%         xlabel ('h');
%         ylabel ('D(h)');
%         title (sprintf ('%s, D(h), %s', meth_str{im}, expstr))
    end
end
%-------------------------------------------------------------------------------
% LOGSCALE DIAGRAMS: CUMULANTS
%-------------------------------------------------------------------------------
if PlotLogScale && CP
    figure (param_display.fig_num + 2); clf
        set (gcf, 'Name', 'Logscale: cumulants')
    nrow = param_est.cum;
    ncol = param_est.cum + 1;  % Number of cumulants for largest order
    idx_curr = 0;  % Index of current cumulant

    for m = 1 : param_est.cum   % rows, cumulant order
        for k = 1 : m + 1  % cols, cumulants of order m
            idx_curr = idx_curr + 1;

            if k == m + 1
                idx_subplot = m * ncol;
            else
                idx_subplot = k + (m - 1) * ncol;
            end

            subplot (nrow, ncol, idx_subplot)

            plot_data.yy        = lgt_mat.cp(idx_curr, :) * loge;
            if doBS
                plot_data.yy_bs = lgt_mat.BS.estB(:, idx_curr, :) * loge;
                plot_data.idx_bs = get_bs_used_j ();
            else
                plot_data.yy_bs = [];
                plot_data.idx_bs = [];
            end
            plot_data.slope     = est_mat.t.cp(idx_curr);
            plot_data.inter     = est_mat.aest.cp(idx_curr);

            plot_logscale (gca, plot_data, param_est, param_bs);
            ylabel (cum_str{idx_curr})
            title (sprintf ('%s %s', meth_str{im_sel}, expstr));
        end
    end
end % if PlotLogScale && CP
%-------------------------------------------------------------------------------
% LOGSCALE DIAGRAMS: STRUCTURE FUNCTIONS
%-------------------------------------------------------------------------------
nrow = 5;
ncol = 5;

iq1_sel = fix (linspace (1, nq1, nrow));
iq2_sel = fix (linspace (1, nq2, ncol));

% if q=2 is not selected, add it replacing closest one:
posq = find (param_est.q == 2);
if ~isempty (posq) && isempty (find (param_est.q(iq1_sel) == 2))
    [~, idx_close] = min (abs (param_est.q(iq1_sel) - 2));
    iq1_sel(idx_close) = posq;
end
posq = find (param_est.q2 == 2);
if ~isempty (posq) && isempty (find (param_est.q2(iq2_sel) == 2))
    [~, idx_close] = min (abs (param_est.q2(iq2_sel) - 2));
    iq2_sel(idx_close) = posq;
end


if PlotLogScale && ZQ
    for imeth = 1 : 2  % Loop DWT and LWT
        if imeth == 1
            lgt_mat_sel = lgt_mat_dwt;
            est_mat_sel = est_mat_dwt;
        else
            lgt_mat_sel = lgt_mat;
            est_mat_sel = est_mat;
        end

        figure (param_display.fig_num + 2 + imeth); clf
        set (gcf, 'Name', 'Logscale: moments')
        for iq1 = 1 : nrow
            for iq2 = 1 : ncol
                subplot (nrow, ncol, iq2 + (iq1 - 1) * ncol)
                plot_data.yy = squeeze (lgt_mat_sel.zq(iq1_sel(iq1), iq2_sel(iq2), :));

                if doBS
                    plot_data.yy_bs = squeeze (lgt_mat_sel.BS.zq(:, iq1_sel(iq1), iq2_sel(iq2), :));
                    plot_data.idx_bs = get_bs_used_j ();
                else
                    plot_data.yy_bs = [];
                    plot_data.idx_bs = [];
                end
                plot_data.slope     = est_mat_sel.t.zq(iq1_sel(iq1), iq2_sel(iq2));
                plot_data.inter     = est_mat_sel.aest.zq(iq1_sel(iq1), iq2_sel(iq2));

                plot_logscale (gca, plot_data, param_est, param_bs);
                ylabel ('log_2 S(q_1,q_2, j)');
                title (sprintf ('%s, %s, q_1=%.2f, q_2=%.2f', ...
                                meth_str{imeth},  expstr, ...
                                param_est.q(iq1_sel(iq1)), ...
                                param_est.q2(iq2_sel(iq2))));
            end
        end
    end
end
%-------------------------------------------------------------------------------
% LOGSCALE DIAGRAMS: SUPCOEF, MINCOEF
%-------------------------------------------------------------------------------
if PlotLogScale
    figure (param_display.fig_num + 5); clf
    set (gcf, 'Name', 'Logscale: hmin')
    for ic = 1 : ncomp
        %-----------------------------------
        %- h_min, coefs
        subplot (ncomp, 3, (ic - 1) * 3 + 1)

        plot_data.yy     = log2 (logstat.DWT.supcoef(ic, :));
        plot_data.slope  = est.DWT.h_min(ic);
        plot_data.inter  = est.DWT.h_min_aest(ic);

        plot_logscale (gca, plot_data, param_est, param_bs);
        ylabel ('log_2 sup [d(j,.)]');
        title (sprintf ('h_{min,%i}, Coeff', ic));

        %-----------------------------------
        %- h_min, leaders
        subplot (ncomp, 3, (ic - 1) * 3 + 2)

        plot_data.yy     = log2 (logstat.LWT.supcoef(ic, :));
        plot_data.slope  = est.LWT.h_min(ic);
        plot_data.inter  = est.LWT.h_min_aest(ic);

        plot_logscale (gca, plot_data, param_est, param_bs);
        ylabel ('log_2 sup [L(j,.)]');
        title (sprintf ('h_{min,%i}, Leaders', ic));

        %-----------------------------------
        %- h_max, leaders
        subplot (ncomp, 3, (ic - 1) * 3 + 3)

        plot_data.yy     = log2 (logstat.LWT.mincoef(ic, :));
        plot_data.slope  = est.LWT.h_max(ic);
        plot_data.inter  = est.LWT.h_max_aest(ic);

        plot_logscale (gca, plot_data, param_est, param_bs);
        ylabel ('log_2 inf [L(j,.)]');
        title (sprintf ('h_{max,%i}, Leaders', ic));

    end
end
%-------------------------------------------------------------------------------
% LOGSCALE DIAGRAMS: CORRELATIONS
%-------------------------------------------------------------------------------
if PlotLogScale && CP && param_est.cum > 1
    figure (param_display.fig_num + 6); clf
    set (gcf, 'Name', 'Logscale: correlation functions')

    c20j = lgt_mat.cp(3, :) * loge;
    c11j = lgt_mat.cp(4, :) * loge;
    c02j = lgt_mat.cp(5, :) * loge;
    cohe = logstat.DWT.coherence(1, :);
    cohe_lead = logstat.LWT.coherence(1, :);

    %-----------------------------------
    %- Wavelet coherence
    plot_data.yy     = cohe;
    plot_data.yy_bs  = [];
    plot_data.idx_bs = [];
    plot_data.slope  = [];
    plot_data.inter  = [];
    plot_data.flag_decorate = false;

    plot_logscale (gca, plot_data, param_est, param_bs);
    limi_coh = max ([1, abs(plot_data.yy)])  + 0.05;

    %-----------------------------------
    %- Multifractal correlation rho_mf
    rhomf = c11j ./ sqrt (abs (c20j .* c02j));

    plot_data.yy     = rhomf;
    plot_data.yy_bs  = [];
    plot_data.idx_bs = [];
    plot_data.slope  = [];
    plot_data.inter  = [];
    plot_data.linespec_estimates = 'rx-';
    plot_data.flag_decorate = false;

    hold on
    plot_logscale (gca, plot_data, param_est, param_bs);

    xlabel ('j')
    ylabel ('\rho')
    legend ('Wavelet coherence', 'Multifractal correlation \rho_{MF}', ...
            'Location', 'best')

    limi_rhomf = max ([1, abs(plot_data.yy)])  + 0.05;
    limi = max (limi_coh, limi_rhomf);
    ylim ([-limi, limi])
end % if PlotLogScale && CP

function [idx] = get_bs_used_j ()
    if logstat.(meth_name{im}).param_bs.flag_bs_range
        % Bootstrap done only in scaling range
        jmin = logstat.(meth_name{im}).param_est.j1;
        jmax = logstat.(meth_name{im}).param_est.j2;
    else
        % Bootstrap done in all scale
        jmin = 1;
        jmax = length (logstat.(meth_name{im}).nj);
    end
    [idx] = jmin : jmax;
end

end  % function make_plots
