function [S_fou, f_fou, S_wav, f_wav] = ...
        wavelet_fourier_spectrum (data, nwt, window_size, fs, fig_num)
%function [S_fou, f_fou, S_wav, f_wav] = ...
%       wavelet_fourier_spectrum (data, nwt, window_size, fs, fig_num)
% Compute and plot the Fourier and wavelet spectrum of data, both as a
% function of frequency.
%
% The Fourier spectrum is computed with a Welch periodogram with 50%
% overlapping windows.
%
% The wavelet spectrum is computed with Daubechies wavelet, using the DxPx1d
% routine in mf_bs_tool.
%
% Inputs:
%   - data: input array (1d)
%   - nwt: number of vanishing moments for the Daubechies wavelet.
%   - window_size: number of samples for the Hamming window used in the Welch
%   periodogram.
%   - fs: sampling frequency of the data. Default: 1.
%   - fig_num: number property of the figure that will be created. If it is
%   0, then no figures are created. Default: 1.
% Outputs:
%   - S_fou: Fourier spectrum
%   - f_fou: array of frequencies for Fourier spectrum coefficients
%   - S_wav: wavelet spectrum
%   - f_wav: array of frequencies for wavelet spectrum coefficients
%
% Roberto Leonarduzzi, Patrice Abry
% June 2018


    if ~exist ('fs', 'var')
        fs = 1;
    end
    if ~exist ('fig_num', 'var')
        fig_num = 1;
    end

    %% Fourier analysis

    S_fou = pwelch (data, window_size, window_size / 2);
    S_fou = S_fou * 4; % Adjust energy for negative frequencies

    f_fou = fs * (0 : 1 / (window_size) : 0.5);

    %% Wavalet analysis

    S_wav = compute_wavelet_spectrum (data, nwt);
    j_wav = 1 : length (S_wav);

    S_wav = S_wav(end : -1 : 1);
    f_wav = 3/4 * fs ./ 2.^(j_wav(end : -1 : 1));


    if fig_num > 0
        figure (fig_num)
        plot (log2 (f_fou), log2 (S_fou), 'k', 'LineWidth', 2)
        hold on
        plot (log2 (f_wav), log2 (S_wav), 'ro-', 'LineWidth', 2)
        hold off
        grid on
        xlabel ('log_2 f')
        ylabel ('log_2 S(f)')
    end
end


function [S] = compute_wavelet_spectrum (data, nwt)
    param_est.nwt    = nwt;
    param_est.gamint = 0.5;  % L2 norm for coefficients
    param_est.p      = inf;
    param_est.sym    = 0;
    param_est.j1_ini = 1;
    param_est.j1     = 1;
    param_est.j2     = 3;
    param_est.wtype  = 0;
    param_est.plead_corr = 0;

    [coef] = DxPx1d (data, param_est);

    for j = length (coef.value) : -1 : 1
        S(j) = mean (coef.value{j} .^ 2);
    end
end
