function [tf, filename_out] = compare_structs (name_orig, orig, name_new, new, ...
                                               verbosity, fields_compare, level, fid)
% function [tf] = compare_structs (orig, new, verbosity, fieldnames, level)
%
% Compares the structures orig and new. If they are not equal, compares their
% subfields and informs which one has differences.
%
% Verbosity levels:
%   - 1: indicates which fields are differet
%   - 2: prints the differences between the fields
%   - 1x: like 1 or 2, but saves output to file.        
%
% RFL, January 2016

%-------------------------------------------------------------------------------
if nargin < 5
    verbosity = 1;
end

if nargin < 6
    fields_compare = [];
end

if nargin < 7
    level = 0;
end

indent = blanks (level * 4);


if verbosity > 10
    flag_write_to_file = true;
    verbosity = verbosity - 10;
else
    flag_write_to_file = false;
end

if level == 0
    if flag_write_to_file
        filename_out = sprintf ('results_comp_%s--%s.txt', name_orig, name_new);
        if level == 0
            fid = fopen (filename_out, 'wt');
        end
    else
        filename_out = '';
        fid = 1;
    end
end
%-------------------------------------------------------------------------------

% fprintf (fid, '%s------------------------------\n', indent);
% fprintf (fid, '%sComparing %s and %s\n', indent, name_orig, name_new);
% fprintf (fid, '\n');

tf = isequaln (orig, new);

% If structs are equal return with no further work
if tf
    % fprintf (fid, '%s%s and %s are the same\n', indent, name_orig, name_new);
    return
end

if isempty (fields_compare)
    fields = fieldnames (orig);
    fields_new = fieldnames (new);
else
    fields = fields_compare;
    fields_new = fields_compare;
end

if ~isequal (fields, fields_new)
    fprintf (fid, '%sFields of %s and %s are different.\n', indent, name_orig, name_new);
    return
end

for k = 1 : length (fields)
    fieldname_orig = sprintf ('%s.%s', name_orig, fields{k});
    fieldname_new = sprintf ('%s.%s', name_new, fields{k});

    if isstruct (orig.(fields{k}))
        compare_structs (fieldname_orig, orig.(fields{k}), ...
                         fieldname_new , new.(fields{k}), ...
                         verbosity, [], level + 1, fid);
    elseif ~isequaln (orig.(fields{k}), new.(fields{k}))
        if verbosity >= 1
            fprintf (fid, 'Fields %s and %s are different.\n', ...
                     fieldname_orig, fieldname_new);
        end
        if verbosity >= 2
            fprintf (fid, '%sDifferences are:\n', indent);
            diff_fun (orig.(fields{k}), new.(fields{k}), indent, fid);
        end
    end
end

fprintf (fid, '%s------------------------------\n', indent);

if flag_write_to_file && level == 0
    fclose (fid);
end

end  % function compare_structs

% Function to display differences:
function diff_fun (orig, new, indent, fid)
% Shows differences between arrays orig and new
    diff_idx = find (orig ~= new);
    for k = 1 : length (diff_idx)
        fprintf (fid, '%sidx=%i, diff=%g\n', indent, diff_idx(k), ...
                 round (log10 (abs (orig(k) - new(k)))));
    end
end
